@@ -28,6 +28,7 @@
 #include <grp.h>
 #include <sys/types.h>
 #include <sys/stat.h>
+#include <sys/uio.h>
 #include <unistd.h>
 #include <string.h>
 
@@ -67,6 +68,10 @@
 #define IPN_SO_DESCR 1
 #endif
 
+#ifndef MIN
+#define MIN(X,Y) (((X)<(Y))?(X):(Y))
+#endif
+
 /* Fallback names for the control socket, NULL-terminated array of absolute
  * filenames. */
 char *fallback_sockname[] = {
@@ -206,8 +211,6 @@ VDECONN *vde_open_real(char *given_sockname, char *descr,int interface_version,
 	if((conn->fddata = socket(AF_IPN_STOLEN,SOCK_RAW,IPN_ANY)) >= 0) {
 		/* IPN_STOLEN service exists */
 		sockun->sun_family = AF_IPN_STOLEN;
-	}
-	if (conn->fddata >= 0) {
 		if (port != 0 || req->type == REQ_NEW_PORT0)
 			setsockopt(conn->fddata,0,IPN_SO_PORT,&port,sizeof(port));
 		/* If we're given a sockname, just try it */
@@ -238,7 +241,8 @@ VDECONN *vde_open_real(char *given_sockname, char *descr,int interface_version,
 			*(conn->inpath.sun_path)=0; /*null string, do not delete "return path"*/
 			conn->fdctl=-1;
 			goto cleanup;
-		}
+		} else
+			close(conn->fddata);
 	}
 #endif
 	if((conn->fdctl = socket(AF_UNIX, SOCK_STREAM, 0)) < 0)
@@ -440,3 +444,101 @@ int vde_close(VDECONN *conn)
 		return -1;
 	}
 }
+
+/* vdestream */
+
+#define MAXPACKET 1521
+
+struct vdestream {
+	void *opaque;
+	int fdout;
+	ssize_t (*frecv)(void *opaque, void *buf, size_t count);
+	void (*ferr)(void *opaque, int type, char *format, ...);
+	char fragment[MAXPACKET];
+	char *fragp;
+	unsigned int rnx,remaining;
+};
+
+VDESTREAM *vdestream_open(void *opaque,
+		int fdout,
+		ssize_t (*frecv)(void *opaque, void *buf, size_t count),
+		void (*ferr)(void *opaque, int type, char *format, ...)
+		)
+{
+	VDESTREAM *vdestream;
+	if ((vdestream=calloc(1,sizeof(struct vdestream)))==NULL) {
+		errno=ENOMEM;
+		return NULL;
+	} else {
+		vdestream->opaque=opaque;
+		vdestream->fdout=fdout;
+		vdestream->frecv=frecv;
+		vdestream->ferr=ferr;
+		return vdestream;
+	}
+}
+
+ssize_t vdestream_send(VDESTREAM *vdestream, const void *buf, size_t len)
+{
+	if (len <= MAXPACKET) {
+		unsigned char header[2];
+		struct iovec iov[2]={{header,2},{(void *)buf,len}};
+		header[0]=len >> 8;
+		header[1]=len & 0xff;
+		return writev(vdestream->fdout,iov,2);
+	} else
+		return 0;
+}
+
+void vdestream_recv(VDESTREAM *vdestream, unsigned char *buf, size_t len)
+{
+	//fprintf(stderr,"%s: splitpacket rnx=%d remaining=%d size=%d\n",myname,rnx,vdestream->remaining,len);
+	if (len==0) return;
+	if (vdestream->rnx>0) {
+		register int amount=MIN(vdestream->remaining,len);
+		//fprintf(stderr,"%s: fragment amount %d\n",myname,amount);
+		memcpy(vdestream->fragp,buf,amount);
+		vdestream->remaining-=amount;
+		vdestream->fragp+=amount;
+		buf+=amount;
+		len-=amount;
+		if (vdestream->remaining==0) {
+			//fprintf(stderr,"%s: delivered defrag %d\n",myname,vdestream->rnx);
+			vdestream->frecv(vdestream->opaque,vdestream->fragment,vdestream->rnx);
+			vdestream->rnx=0;
+		}
+	}
+	while (len > 0) {
+		vdestream->rnx=(buf[0]<<8)+buf[1];
+		len-=2;
+		//fprintf(stderr,"%s %d: packet %d size %d %x %x\n",myname,getpid(),vdestream->rnx,len,buf[0],buf[1]);
+		buf+=2;
+		if (vdestream->rnx > MAXPACKET) {
+			if (vdestream->ferr != NULL)
+				vdestream->ferr(vdestream->opaque,PACKET_LENGTH_ERROR,
+						"size %d expected size %d",len,vdestream->rnx);
+			vdestream->rnx=0;
+			return;
+		}
+		if (vdestream->rnx > len) {
+			//fprintf(stderr,"%s: begin defrag %d\n",myname,vdestream->rnx);
+			vdestream->fragp=vdestream->fragment;
+			memcpy(vdestream->fragp,buf,len);
+			vdestream->remaining=vdestream->rnx-len;
+			vdestream->fragp+=len;
+			len=0;
+		} else {
+			//fprintf(stderr,"%s: deliver %d\n",myname,vdestream->rnx);
+			vdestream->frecv(vdestream->opaque,buf,vdestream->rnx);
+			buf+=vdestream->rnx;
+			len-=vdestream->rnx;
+			vdestream->rnx=0;
+		}
+	}
+}
+
+void vdestream_close(VDESTREAM *vdestream)
+{
+	free(vdestream);
+}
+
